__all__ = ('AbstractRunner',  'CoroEvent', 'CoroContext', 'CoroStatusChanged')

from abc import ABC, abstractmethod
from typing import Any, Callable, Coroutine, TypeVar

from v6d2ctx.integration.event import *
from v6d2ctx.integration.responsetype import *

T = TypeVar('T')


class CoroEvent(Event):
    pass


class CoroContext:
    def __init__(self, events: SendableEvents[CoroEvent]) -> None:
        self.events = events


class CoroStatusChanged(CoroEvent):
    def __init__(self, status: ResponseType) -> None:
        self.status = status

    def json(self) -> ResponseType:
        return {'status': self.status}


class AbstractRunner(ABC):
    @abstractmethod
    async def run(self, coro: Coroutine[Any, Any, T], /) -> T:
        raise NotImplementedError

    @abstractmethod
    async def runctx(self, ctxcoro: Callable[[CoroContext], Coroutine[Any, Any, T]], /) -> T:
        raise NotImplementedError
